@@ -21,7 +21,7 @@
 extern "C" {
 #endif
 
-#define GTK_TYPE_FORM		       (gtk_form_get_type ())
+#define GTK_TYPE_FORM		       (gui_gtk_form_get_type ())
 #ifdef USE_GTK3
 #define GTK_FORM(obj)		       (G_TYPE_CHECK_INSTANCE_CAST((obj), GTK_TYPE_FORM, GtkForm))
 #define GTK_FORM_CLASS(klass)	       (G_TYPE_CHECK_CLASS_CAST((klass), GTK_TYPE_FORM, GtkFormClass))
@@ -53,33 +53,28 @@ struct _GtkFormClass
 };
 
 #ifdef USE_GTK3
-GType gtk_form_get_type(void);
+GType gui_gtk_form_get_type(void);
 #else
-GtkType gtk_form_get_type(void);
+GtkType gui_gtk_form_get_type(void);
 #endif
 
-GtkWidget *gtk_form_new(void);
+GtkWidget *gui_gtk_form_new(void);
 
-void gtk_form_put(GtkForm * form, GtkWidget * widget,
-	gint x, gint y);
+void gui_gtk_form_put(GtkForm * form, GtkWidget * widget, gint x, gint y);
 
-void gtk_form_move(GtkForm *form, GtkWidget * widget,
-	gint x, gint y);
+void gui_gtk_form_move(GtkForm *form, GtkWidget * widget, gint x, gint y);
 
-void gtk_form_move_resize(GtkForm * form, GtkWidget * widget,
-	gint x, gint y,
-	gint w, gint h);
+void gui_gtk_form_move_resize(GtkForm * form, GtkWidget * widget, gint x, gint y, gint w, gint h);
 
-/* These disable and enable moving and repainting respectively.  If you
- * want to update the layout's offsets but do not want it to repaint
- * itself, you should use these functions.
- */
+// These disable and enable moving and repainting respectively.  If you
+// want to update the layout's offsets but do not want it to repaint
+// itself, you should use these functions.
 
-void gtk_form_freeze(GtkForm *form);
-void gtk_form_thaw(GtkForm *form);
+void gui_gtk_form_freeze(GtkForm *form);
+void gui_gtk_form_thaw(GtkForm *form);
 
 
 #ifdef __cplusplus
 }
 #endif
-#endif	/* __GTK_FORM_H__ */
+#endif	// __GTK_FORM_H__
